@@ -34,6 +34,22 @@ import random
 import string
 from datetime import timedelta
 
+import sys
+from dataclasses import dataclass
+from enum import Enum
+from collections import defaultdict
+
+try:
+    from pandas import DataFrame  # type: ignore
+    import pandas as pd
+except ImportError:
+    DataFrame = None
+try:
+    from graphviz import Digraph  # type: ignore
+except ImportError:
+    Digraph = None
+
+
 FilenameLookup = Dict[str, str]
 
 log: logging.Logger = logging.getLogger("resotoclient")
@@ -327,7 +343,7 @@ class ResotoClient:
             raise AttributeError(response.text)
 
     def search_list(
-        self, search: str, section: Optional[str] = None, graph: str = "resoto"
+        self, search: str, section: Optional[str] = "reported", graph: str = "resoto"
     ) -> Iterator[JsObject]:
         params = {}
         if section:
@@ -342,7 +358,7 @@ class ResotoClient:
             raise AttributeError(response.text)
 
     def search_graph(
-        self, search: str, section: Optional[str] = None, graph: str = "resoto"
+        self, search: str, section: Optional[str] = "reported", graph: str = "resoto"
     ) -> Iterator[JsObject]:
         params = {}
         if section:
@@ -356,10 +372,13 @@ class ResotoClient:
             raise AttributeError(response.text)
 
     def search_aggregate(
-        self, search: str, graph: str = "resoto"
+        self, search: str, section: Optional[str] = "reported", graph: str = "resoto"
     ) -> Iterator[JsObject]:
+        params = {}
+        if section:
+            params["section"] = section
         response = self._post(
-            f"/graph/{graph}/search/aggregate", data=search, stream=True
+            f"/graph/{graph}/search/aggregate", params=params, data=search, stream=True
         )
         if response.status_code == 200:
             return map(lambda line: json_loadb(line), response.iter_lines())
@@ -476,7 +495,7 @@ class ResotoClient:
         headers = headers or {}
         if not files:
             headers["Content-Type"] = "text/plain"
-            body = command
+            body = command.encode("utf-8")
         else:
             headers["Resoto-Shell-Command"] = command
             headers["Content-Type"] = "multipart/form-data; boundary=file-upload"
@@ -642,8 +661,216 @@ class ResotoClient:
         else:
             raise AttributeError(response.text)
 
+    def dataframe(self, search: str, section: Optional[str] = "reported", graph: str = "resoto", flatten: bool = True) -> DataFrame:  # type: ignore
+        if DataFrame is None:
+            raise ImportError("Python package resotoclient[extras] is not installed")
+        aggregate_search = False
+
+        if search.startswith("aggregate"):
+            aggregate_search = True
+            iter = self.search_aggregate(search=search, section=section, graph=graph)
+        else:
+            iter = self.search_list(search=search, section=section, graph=graph)
+
+        def extract_node(node: JsObject) -> Optional[JsObject]:
+            node_data = node
+            if not isinstance(node_data, dict):
+                return None
+            if aggregate_search:
+                if (
+                    flatten
+                    and "group" in node_data
+                    and isinstance(node_data["group"], dict)
+                ):
+                    group = node_data["group"]
+                    del node_data["group"]
+                    for k, v in group.items():
+                        node_data[k] = v
+            else:
+                if flatten:
+                    if not "reported" in node or not isinstance(node["reported"], dict):
+                        return None
+                    node_data = node["reported"]
+                    for k, v in node.items():
+                        if isinstance(v, dict) and k != "reported":
+                            node_data[k] = v
+                    node_data["cloud_id"] = js_find(
+                        node,
+                        ["ancestors", "cloud", "reported", "id"],
+                    )
+                    node_data["cloud_name"] = js_find(
+                        node,
+                        ["ancestors", "cloud", "reported", "name"],
+                    )
+                    node_data["account_id"] = js_find(
+                        node,
+                        ["ancestors", "account", "reported", "id"],
+                    )
+                    node_data["account_name"] = js_find(
+                        node,
+                        ["ancestors", "account", "reported", "name"],
+                    )
+                    node_data["region_id"] = js_find(
+                        node,
+                        ["ancestors", "region", "reported", "id"],
+                    )
+                    node_data["region_name"] = js_find(
+                        node,
+                        ["ancestors", "region", "reported", "name"],
+                    )
+                    node_data["zone_id"] = js_find(
+                        node,
+                        ["ancestors", "zone", "reported", "id"],
+                    )
+                    node_data["zone_name"] = js_find(
+                        node,
+                        ["ancestors", "zone", "reported", "name"],
+                    )
+            return node_data
+
+        nodes = [extract_node(node) for node in iter]
+        return pd.json_normalize(nodes)  # type: ignore
+
+    def graphviz(
+        self,
+        search: str,
+        section: Optional[str] = "reported",
+        graph: str = "resoto",
+        engine: str = "sfdp",
+        format: str = "svg",
+    ) -> Digraph:  # type: ignore
+        if Digraph is None:
+            raise ImportError("Python package resotoclient[extras] is not installed")
+
+        digraph = Digraph(comment=search)
+        digraph.format = format
+        digraph.engine = engine
+        digraph.graph_attr = {"rankdir": "LR", "splines": "true", "overlap": "false"}  # type: ignore
+        digraph.node_attr = {  # type: ignore
+            "shape": "plain",
+            "colorscheme": "paired12",
+        }
+        cit = iter(range(0, sys.maxsize))
+        colors: Dict[str, int] = defaultdict(lambda: (next(cit) % 12) + 1)
+
+        results = self.search_graph(search=search, section=section, graph=graph)
+
+        for elem in results:
+            if elem.get("type") == "node":
+                kind = js_get(elem, ["reported", "kind"])
+                color = colors[kind]
+                rd = ResourceDescription(
+                    id=js_get(elem, ["reported", "id"]),
+                    name=js_get(elem, ["reported", "name"]),
+                    uid=js_get(elem, ["id"]),
+                    kind=parse_kind(kind),
+                    kind_name=kind,
+                )
+                digraph.node(  # type: ignore
+                    name=js_get(elem, ["id"]),
+                    # label=rd.name,
+                    label=render_resource(rd, color),
+                    shape="plain",
+                )
+            elif elem.get("type") == "edge":
+                digraph.edge(js_get(elem, ["from"]), js_get(elem, ["to"]))  # type: ignore
+
+        return digraph
+
 
 def rnd_str(str_len: int = 10) -> str:
     return "".join(
         random.choice(string.ascii_uppercase + string.digits) for _ in range(str_len)
     )
+
+
+def js_find(node: JsObject, path: List[str]) -> Optional[str]:
+    """
+    Get a value in a nested dict.
+    """
+    if len(path) == 0:
+        return None
+    else:
+        value = node.get(path[0])
+        if len(path) == 1:
+            return value if isinstance(value, str) else None
+        if not isinstance(value, dict):
+            return None
+        return js_find(value, path[1:])
+
+
+def js_get(node: JsObject, path: List[str]) -> str:
+    result = js_find(node, path)
+    if result is None:
+        raise ValueError(f"Path {path} not found in {node}")
+    return result
+
+
+class ResourceKind(Enum):
+    UNKNOWN = 1
+    INSTANCE = 2
+    VOLUME = 3
+    IMAGE = 4
+    FIREWALL = 5
+    K8S_CLUSER = 6
+    NETWORK = 7
+    LOAD_BALANCER = 8
+    CLOUD = 9
+
+
+do_kinds = {
+    "droplet": ResourceKind.INSTANCE,
+    "volume": ResourceKind.VOLUME,
+    "image": ResourceKind.IMAGE,
+    "firewall": ResourceKind.FIREWALL,
+    "kubernetes_cluster": ResourceKind.K8S_CLUSER,
+    "network": ResourceKind.NETWORK,
+    "load_balancer": ResourceKind.LOAD_BALANCER,
+}
+
+
+def parse_kind(kind: str) -> ResourceKind:
+    cloud, rest = kind.split("_")[0], "_".join(kind.split("_")[1:])
+    if cloud == "digitalocean":
+        return do_kinds.get(rest) or ResourceKind.UNKNOWN
+    else:
+        return ResourceKind.UNKNOWN
+
+
+kind_colors = {
+    ResourceKind.INSTANCE: "8",
+    ResourceKind.VOLUME: "4",
+    ResourceKind.IMAGE: "7",
+    ResourceKind.FIREWALL: "6",
+    ResourceKind.K8S_CLUSER: "5",
+    ResourceKind.NETWORK: "10",
+    ResourceKind.LOAD_BALANCER: "9",
+    ResourceKind.CLOUD: "1",
+}
+
+
+@dataclass
+class ResourceDescription:
+    uid: str
+    name: str
+    id: str
+    kind: ResourceKind
+    kind_name: str
+
+
+def render_resource(
+    resource: ResourceDescription,
+    color: int,
+) -> str:
+    return f"""\
+<<TABLE STYLE="ROUNDED" COLOR="{color}" BORDER="3" CELLBORDER="1" CELLPADDING="5">
+    <TR>
+        <TD SIDES="B">{resource.kind_name}</TD>
+    </TR>
+    <TR>
+        <TD SIDES="B">{resource.id}</TD>
+    </TR>
+    <TR>
+        <TD BORDER="0">{resource.name}</TD>
+    </TR>
+</TABLE>>"""
